# -*- coding: utf-8 -*-
import datetime
from odoo import api, models
from odoo.tools import misc, config, DEFAULT_SERVER_DATETIME_FORMAT
from odoo.addons.saas_worker.controllers.main import CREATION_PREFIX
from odoo.addons.saas_worker.const import EXTRA_APPS, TEST, WEBSITE
from odoo.addons.saas_worker.util import get_size
from odoo.addons.saas_worker import metabase



if TEST:
    # force ping to right server
    config['publisher_warranty_url'] = WEBSITE + '/publisher-warranty/'

class publisher_warranty_contract(models.AbstractModel):
    _inherit = "publisher_warranty.contract"

    @api.model
    def _inject_saas_data(self, msg):
        Modules = self.env['ir.module.module']
        has_website = Modules.search_count([('name', '=', 'website'), ('state', '=', 'installed')]) == 1
        homepage = self.env.ref('website.homepage', raise_if_not_found=False)
        odoo_oauth_provider = self.env.ref('auth_oauth.provider_openerp', raise_if_not_found=False)
        if homepage:
            homepage_md = homepage.get_metadata()[0]
            website_homepage_modified = (
                homepage_md.get('write_date') is not None and
                homepage_md.get('create_date') != homepage_md.get('write_date'))
            msg['website_homepage_modified'] = website_homepage_modified

        apps_domain = ['|', ('application', '=', True), ('name', 'in', EXTRA_APPS),
                  ('state', 'in', ['installed', 'to upgrade', 'to remove'])]
        apps = Modules.search_read(apps_domain, ['name'])
        user_domain = [('share', '=', False)]
        Users = self.env['res.users']
        user_fields = [field for field in ['name', 'login', 'password_crypt', 'oauth_uid', 'oauth_provider_id', 'odoo_com_uid', 'login_date', 'groups_id'] if field in Users._fields]
        users = Users.search_read(user_domain, user_fields, order='id')
        group_system_id = self.env.ref('base.group_system').id
        for user in users:
            if not user['odoo_com_uid'] and odoo_oauth_provider and user['oauth_uid'] and user['oauth_provider_id'][0] == odoo_oauth_provider.id:
                user['odoo_com_uid'] = user['oauth_uid']
                user['oauth'] = True
            user.pop('oauth_uid', False)
            user.pop('oauth_provider_id', False)
            if isinstance(user['login_date'], (datetime.datetime)):
                user['login_date'] = user['login_date'].strftime(DEFAULT_SERVER_DATETIME_FORMAT)
            user['is_admin'] = group_system_id in user.pop('groups_id', [])
        dbname = msg['dbname']
        if dbname.startswith(CREATION_PREFIX):
            msg['dbname'] = dbname[len(CREATION_PREFIX):]

        (backup_size, fs_size) = get_size(dbname)
        ICP = self.env['ir.config_parameter']

        companies = self.env['res.company'].with_context(active_test=True).search([], order='id asc')
        main_company_id = companies[:1]
        msg.update({
            "website": has_website,
            "apps": [app['name'] for app in apps],
            "users": users,
            "backup_size": misc.human_size(backup_size),
            "fs_size": misc.human_size(fs_size),
            "total_size": backup_size + fs_size,
            "main_company_country": main_company_id and main_company_id.country_id and main_company_id.country_id.code or False,
            "company_count": len(companies),
            "last_rpc_call": ICP.get_param('database.rpc', False),
        })
        if self.env['ir.model.data'].search_count([('module', '=', 'base_automation'), ('name', '=', 'installed-17')]):
            msg['has_base_automation'] = True

        if ICP.get_param('database.is_neutralized', False):
            msg['is_neutralized'] = True

        parent_uuid = ICP.get_param('origin.database.uuid', False)
        if parent_uuid:
            msg['parent_uuid'] = parent_uuid

        if ICP.get_param('saas.db_purpose') == 'support':
            # pings are already taking a lot of space in accounts db, so we send the db purpose only when it's for support
            # since this is for now the only case where it does something
            msg['db_purpose'] = 'support'

        mdb = metabase.get(dbname)

        # Always force ping from trial database
        # This is necessary because we want updated data to create accurate contracts.
        # Just before the proposal, we force a ping from the user database
        msg['force_ping'] = mdb and mdb.mode == 'trial' or self._context.get('force_ping')

        if mdb and mdb.last_cnx_date:
            msg['last_cnx_date'] = mdb.last_cnx_date.strftime(misc.DEFAULT_SERVER_DATETIME_FORMAT)
        if mdb and mdb.expire_date:
            msg['expire_date'] = mdb.expire_date.strftime(misc.DEFAULT_SERVER_DATETIME_FORMAT)
        if mdb:
            msg['extra_apps'] = mdb.extra_apps

    @api.model
    def _get_message(self):
        msg = super(publisher_warranty_contract, self)._get_message()
        self._inject_saas_data(msg)
        msg.pop('enterprise_code', None)
        return msg
